#include "vsi-dummy-srv.h"

int shem_init(unsigned long size);

void Init_data(void* data, size_t size){
  wchar_t wc;
  memset(&wc,0xe4,sizeof(wc));
  wmemset((wchar_t*)data,wc,size/sizeof(wc));
}

void Usage(){
  cerr << "Usage: " << me << " [-p <port>] "   << endl;
  cerr << "Providing Dummy data to Client" << endl;
  exit(1);
}

// Binding and Listening
int Listener(Socket* sock, const string &ip, int port){
  int status;
  
  fprintf(stderr,"Binding Socket to %s:%d \n", ip.c_str(), port);
  if( sock->bind(ip,port) == -1){
    fprintf(stderr,"Error: %s bind failed in Listener()\n", me);
    return 0;
  }
  fprintf(stderr,"Listening to sender socket ...\n");
  if( sock->listen() == -1){
    fprintf(stderr,"Error: %s listen failed in Listener()\n", me);
    return 0;
  }else{
    fprintf(stderr,"Successed\n");
    fprintf(stderr,"%s DEBUG: Waiting for connection with port(%d)\n", me, port);
  }

  return 1;

}


int  main(int argc, char *argv[]) {
 /* *** Initialize *** */
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  unsigned short port=7000;
  int mtu=1500;

  /* * Parse command-line options, if any * */ 
  int k = 0; 
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hp:")) { 
    case 'h' : // Help 
      Usage();
      break;
    case 'p': 
      port = atoi(optarg);
      break;
    default:
      break;
    }
  }

  string srv_ip("0.0.0.0");
  int status=0;
  Socket *out_sock = new TCPSocket(mtu);
  
  if( Listener(out_sock, srv_ip, port) ==0){
    fprintf(stderr, "%s Listener Failed.\n", me);
    delete out_sock;
    exit(1);
  }

  // Shard Memory  Initialize
  unsigned long shm_size = 1024;
  int shemid=-1;
  if((shemid = shem_init(shm_size)) == -1){  exit(1); 
  }else{
    fprintf(stderr,"Shaed memID = %d\n", shemid);
  }

  ThreadDMY* threader = new ThreadDMY(*out_sock);  
  threader->Set_shem(shemid, shm_size);

  fprintf(stderr,"%s DEBUG: Thread generating from main\n",me);
  threader->proc();
  
  char sRecvBuf[256];
  while(status != -1){
    fprintf(stderr,"\n Nclient=%d, ", threader->NClient());
    fprintf(stderr,"Terminate by \"QUIT\" \n");
    scanf("%s", sRecvBuf);
    StrUpper(sRecvBuf,strlen(sRecvBuf));
    if(strncmp(sRecvBuf,"QUIT",4)==0){
      break;
    }
  }

  if(shmctl(shemid,IPC_RMID,0)==-1){
    perror("shmctl");
  }
  delete out_sock;
  delete threader;

}


 // Shard Memory  Initialize
int shem_init(unsigned long size){

  int id;
  if((id=shmget(IPC_PRIVATE,size,IPC_CREAT|0666))==-1){
    perror("shmget");
    return(-1);
  }
  char* adr;
  if((adr = (char*)shmat(id,0,0)) == (char*)-1){
    perror("shmget");
    return(-1);
  }
  Init_data(adr, size);
  return(id);
}
