/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2003 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */

#ifndef RRD_H
#define RRD_H

using namespace std;

#include <sys/types.h>
#include <string>
#include <fstream>
#include <sys/time.h>

struct Uint16Sample 
{
	 double _time;
	 u_int16_t _val;
};

struct Uint32Sample 
{
	 double _time;
	 u_int32_t _val;
};

struct DoubleSample 
{
	 double _time;
	 double _val;
};

struct Uint16RRD
{
	 Uint16Sample* _db;
	 u_int32_t _sz;
	 string _file_name;
	 u_int32_t _idx;
	 Uint16RRD(const u_int32_t sz, const char* file_name);
	 ~Uint16RRD();
	 inline void update(const u_int16_t& val) {
		  struct timeval tv_now;
		  gettimeofday(&tv_now, NULL);
		  _db[_idx]._time=tv_now.tv_sec+tv_now.tv_usec/1e6;
		  _db[_idx]._val=val;
		  _idx=(_idx+1)%_sz;
	 }
};

struct Uint32RRD
{
	 Uint32Sample* _db;
	 u_int32_t _sz;
	 string _file_name;
	 u_int32_t _idx;
	 Uint32RRD(const u_int32_t sz, const char* file_name);
	 ~Uint32RRD();
	 inline void update(const u_int32_t& val) {
		  struct timeval tv_now;
		  gettimeofday(&tv_now, NULL);
		  _db[_idx]._time=tv_now.tv_sec+tv_now.tv_usec/1e6;
		  _db[_idx]._val=val;
		  _idx=(_idx+1)%_sz;
	 }
};

struct DoubleRRD
{
	 DoubleSample* _db;
	 u_int32_t _sz;
	 string _file_name;
	 u_int32_t _idx;
	 DoubleRRD(const u_int32_t sz, const char* file_name);
	 ~DoubleRRD();
	 inline void update(const double& val) {
		  struct timeval tv_now;
		  gettimeofday(&tv_now, NULL);
		  _db[_idx]._time=tv_now.tv_sec+tv_now.tv_usec/1e6;
		  _db[_idx]._val=val;
		  _idx=(_idx+1)%_sz;
	 }
};


#endif // RRD_H
