/*
 *  VRClientSocket.h
 *
 *  Created by David Lapsley on Mon Feb 23 2004.
 *
 */

/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2003 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */

#ifndef VRCLIENTSOCKET_H
#define VRCLIENTSOCKET_H

#include <common.h>
#include <Socket.h>
#include <VRSocket.h>
#include <string>
#include <iostream>
#include <syslog.h>

/** VRClientSocket class
 * This class implements an RTP server socket. It inherits from VRClientSocketA.
 */
class VRClientSocket: public VRSocket
{
 public:
  /** Processing function.
   * This function is the entry point for RTCP processing in this class. 
   * It runs in the calling mode context.
   */
  void rtp_proc();
  /** Processing function.
   * Start RTP thread execution.
   */
  void rtcp_proc();
  /** Thread callback.
   * This function is called when the base class method "thread_create()" is
   * called. This results in thread_func() being executed in it's own
   * thread. Depending on the mode, either recv_proc() or send_proc() will
   * be called by thread_func(). This means that recv_proc() or send_proc()
   * will be running in their own thread of execution.
   */
  void* thread_func();
  /** Constructor.
   * This constructor creates a socket and sets the internal MTU member
   * to be equal to the supplied parameter.
   * @param in_sock input socket reference.
   * @param out_sock output socket reference.
   * @param rtcp_sock rtcp socket reference.
   * @param tvg_duration Duration of test vector generation.
   * @param mtu size of packet payload in bits.
   * @param rtcp_interval maximum interval between transmission of RTCP packets.
   */
  VRClientSocket(Socket& in_sock,
		 Socket& out_sock,
		 RTCPSocket& rtcp_sock,
		 const double& tvg_duration, 
		 const u_int32_t& mtu, 
		 const double& rtcp_interval);
  /** Copy constructor.
   * Copies values of other VRClientSocket into this object.
   * @param s VRClientSocket to copy.
   */
  VRClientSocket(const VRClientSocket &s);
  /** Destructor. */
  virtual ~VRClientSocket();
};

ostream& operator<<(ostream& os, const VRClientSocket& s);


#endif

